{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit DIMMEDN;

{ 
Implementation of the indices depending on an election (those
of the module Dimensi): electoral disproportionality, party
dimensions but the volatility.


Assumption:
 1. The party data are given by the percentages of parties and seats obtained
in the studied election.

 2. The electoral data array will be ordered decreasingly according to the votes.
    Notice tha this order does not induce an order for the percentages of seats.
}

{$N+,E-,F+,O+}

INTERFACE

uses COMUN, DIMDEFN;


procedure Medidas(var camino: TcaminFi;  var gout: Tfichero;
                 Pa: PVecParty;  nodat: TRgPartido;
                 var totv, tote: Tnumero);
{
This procedure calculates the political indices and also saves their scores into
a binary file.
}


procedure MedidasTXT(var camino: TcaminFi; var f: text;
                   Pa: PVecParty;  nodat: TRgPartido;
                   var totv, tote: Tnumero);
{
This procedure calculates the political indices and also saves their scores into
a text file.
}

procedure MedidasHTML(var camino: TcaminFi;  var f: text;
                   Pa: PVecParty;   nodat: TRgPartido;
                   var totv, tote: Tnumero);
{
This procedure calculates the political indices and also saves their scores into
a HTML file.
}


procedure MedidasRS(var camino: TcaminFi;  var f: text;
                  Pa: PVecParty; nodat: TRgPartido;
                  var totv, tote: Tnumero);
{
This procedure calculates the political indices and also saves their scores into
a R source file.
}

                   
IMPLEMENTATION


uses DIMVECTO, DIMSIGLA, UHTML, COMUTIL;

procedure iniDisproportion(Pa: PVecParty; var pD: PVectoR; ndat: TRgPartido);
var i: TRgPartido;
Begin
  DimVectoR(pD, ndat);
  for i:= 0 to ndat do pD^[i]:= Pa^[i].Ep - Pa^[i].Vp;
End;

function Chi2Saint(Pa: PVecParty; pD: PVector; nodat: TRgPartido): Tnumero;

{
 It calculates the Sainte-Lague's index of disproportionality, 

  \sum_(i=0)^nodat (V[i] - E[i])^2 /V[i]

 This index can be viewed as the fit goodness Chi-square statistic for the 
 following distributions:
 * expected distribution =  distribution of votes
 * observed distribution = distribution of seats

}
  var ac: Tnumero2;
      i: TRgPartido;

Begin
  ac:= 0;

  for i:= 0 to nodat do
    begin
      if Pa^[i].Ep > 0
      then
          ac:= ac + SQR(pD^[i])/Pa^[i].Vp
      else
          ac:= ac + Pa^[i].Vp;
    end;

  Chi2Saint:= ac;
End;



function DsprRae(pD: PVector; nodat: TRgPartido): Tnumero;

{ 
It calculates the Rae's index of disproportionality, 
      \sum_(i=0)^nodat | V[i] - E[i] | / (nodat+1)
}

  var
      i: TRgPartido;
      ac: Tnumero2;

Begin
  ac:= 0;
  for i:= 0 to nodat do ac := ac + ABS(pD^[i]);

  DsprRae := ac/Succ(nodat);
End;


function BuscaUmbral(var ip: TRgPartido;  nodat: TRgPartido; cota: Tnumero;
                     Pa: PVecParty): boolean;

{
  Given
        "nodat" = number of parties,
         "cota" = vote threshold in percentage,
         "Pa" =   party data vector,

 it searches for "ip" which is the maximum index satisfying that

     V[l] > cota ,      for each  l = 0, ..., "ip"

 If BuscaUmbral = TRUE, then an error arised searching for the index "ip".
 }

label finaliza;

Begin
  ip:= 0;

  while ( Pa^[ip].Vp > cota ) do
    begin
      if ip = nodat then goto finaliza;
      Inc(ip);
    end;

  if ip > 0 
  then 
    Dec(ip)
  else 
    begin
      {ip:= 0;  - unlikely }
      BuscaUmbral:= TRUE;
      exit;      
    end;

  finaliza:
  BuscaUmbral:= FALSE;
End;


function DsprRaeCo(Pa: PVecParty; pD: PVector; nodat: TRgPartido): Tnumero;
{ 
 It calculates the modified Rae index of disproportionality
      \sum_[i=0]^I | V[i] - E[i] | / (I+1)

 where I is the number of parties satisfying that their percentages
 of vote are greater than 0.5%. 
}

var ac: Tnumero2;
    i, ip: TRgPartido;

Begin
  if BuscaUmbral(ip, nodat, 0.5, Pa) then
    begin
      DsprRaeCo:= 999999.0;  { it means not available }
      exit;
    end;

  ac:= 0;
  for i:= 0 to ip do ac := ac + ABS(pD^[i]);

  DsprRaeCo:= ac/Succ(ip);
End;


function DsprLooHan(pD: PVector;  nodat: TRgPartido): Tnumero;

{
It calculates the Loosemore and Hanby index of disproportionality % =
    \sum_(i=0)^(nodat) | V[i] - E[i] | / 2 
}

  var
      i: TRgPartido;
      ac: Tnumero2;

Begin
  ac:= 0;
  for i:= 0 to nodat do ac := ac + ABS(pD^[i]);

  DsprLooHan:= 0.5 * ac;
End;


function DsprGrofman(votos: boolean; Pa: PVecParty; pD: PVector;
               nodat: TRgPartido): Tnumero;

{
It calculates the Grofman's index of disproportionality
  Gro# = \sum_(i=0)^(nodat) | V_p (i) - E_p (i) | /  ENP# .

ENP# is the Laakso & Taagepera's index of the effective number of parties,
given by

 ENP#^(-1) = \sum_(i=0)^(nodat) p#(i)^2,

where p#_i stands for the proportion of votes or seats.
Indeed,
  votos=TRUE (false) means the electoral (parliam.) version of this index.

}

  var i: TRgPartido;
      totx2, ac: Tnumero2;
Begin
  totx2:= 0;    
  ac:= 0;

  for i:= 0 to nodat do
    begin
      if votos
      then 
        totx2:= totx2 + SQR(Pa^[i].Vp * 0.01)
      else
        totx2:= totx2 + SQR(Pa^[i].Ep * 0.01);

      ac:= ac + ABS(pD^[i]);
    end;   

  DsprGrofman:= totx2 * ac;
End;


procedure DsprLijMaxD(pD: PVector;  nodat: TRgPartido;
                      var maximo: Tnumero; var ip: TRgPartido);

{
It calculates the Lijphart's index of disproportionality given by the largest deviation,
"max", between the shares of vote and seat. The order of that party having 
the largest deviation (associated to "maximo") is assigned to "ip".
This means that
 maximo =  Lmax = | V[ip-1] - E[ip-1]  | = max_(i=0,..,nodat) | V[i] - E[i]  |

At the end, "ip" is incremented in an unity.
}

  var i: TRgPartido;

Begin
  maximo:= 0;

  for i:= 0 to nodat do
    if maximo < ABS(pD^[i]) then
      begin
        maximo:= ABS(pD^[i]);
        ip:= i;
      end;

  Inc(ip);
End;


function DsprMCGa(pD: PVector;  nodat: TRgPartido): Tnumero;

{ 
 It calculates the Gallagher's index of disproportionality, which is also
 known as the least squares index,
   Gcm = SQRT( 0.5 *  \sum_i ( V[i] - E[i] )^2  ) 
    
 }

  var ac: Tnumero2;
      i: TRgPartido;

Begin
  ac:= 0;
  for i:= 0 to nodat do ac := ac + SQR(pD^[i]);
  DsprMCGa:= SQRT( ac * 0.5 );
End;


function DsprMCLij(Pa: PVecParty; pD: PVector; nodat: TRgPartido): Tnumero;

{ 
 It calculates the least squares index proposed by Lijphart,
    Lcm = SQRT( 0.5 *  \sum_(i=0)^I ( V[i] - E[i]  )^2  ) 

 This is a modified version of the Gallagher index.    
}

  var ac: Tnumero2;
      i, ip: TRgPartido;

Begin
  if BuscaUmbral(ip, nodat, 0.5, Pa) then
    begin
      DsprMCLij:= 999999.0;
      exit;
    end;

  ac:= 0;
  for i:= 0 to ip do ac := ac + SQR(pD^[i]);
  DsprMCLij:= SQRT( ac * 0.5 );
End;


function DsprRobuL1(Pa: PVecParty;  nodat: TRgPartido;
                    var totv, tote: Tnumero): Tnumero;

{
The robust L1-norm based index of disproportionality, which
was proposed by Pennisi (1998).
}

  var ac: Tnumero2;
      i: TRgPartido;

Begin
  ac:= 0;
  for i:= 0 to nodat do
    begin
      if Pa^[i].Ep > 0
      then
          ac:= ac + ABS( (Pa^[i].Ep / Pa^[i].Vp) - 1 )
      else
          ac:= ac + 1;
    end;

  { ac:= ac*tote/totv; }
  DsprRobuL1:= ac;
End;


function DsprRobuL2(Pa: PVecParty;  nodat: TRgPartido;
                    var totv, tote: Tnumero): Tnumero;

{
The robust L2-norm based index of disproportionality, which
was proposed by Pennisi (1998).
}

  var ac: Tnumero2;
      i: TRgPartido;

Begin
  ac:= 0;
  for i:= 0 to nodat do
    begin
      if Pa^[i].Ep > 0
      then
         ac:= ac + SQR((Pa^[i].Ep / Pa^[i].Vp) - 1 )
      else
         ac:= ac + 1;
    end;

  { ac:= ac*SQR(tote/totv); }
  DsprRobuL2:= ac;
End;


procedure DsprRobuLoo(Pa: PVecParty;  nodat: TRgPartido; var ip: TRgPartido;
                      var totv, tote, maximo: Tnumero);

{
The robust Loo-norm based index of disproportionality, which
was proposed by Pennisi (1998). The return values are given
by
 maximo = the Loo index of disproportionality and
 ip = the party rank index where "maximum" is achieved
}
  var r: Tnumero;
      i: TRgPartido;

Begin
  maximo:= 0;

  for i:= 0 to nodat do
    begin
      if Pa^[i].Ep > 0
      then
          r:= ABS( (Pa^[i].Ep / Pa^[i].Vp) - 1 )
      else
          r:= 1;

      if maximo < r then
        begin
          maximo:= r;
          ip:= i;
        end;
    end;

  { maximo:= maximo*tote/totv; }
  Inc(ip);
End;


procedure modLineal(var SumE, SumV: Tnumero;  m: TRgPartido;
                   Pa: PVecParty;  ajustado: boolean);

{ 
 It estimates the slope in the linear regression model given

     E[i]  =  a  +  b * V[i]  +  Error[i],    \forall i = 0,1,..,m

by using the least squares method. The estimation of "b" is 
derived from the electoral data of the "m"+1 most voted parties.
It returns 
  SumE : the estimation of b,
  SumV : the R2 coefficient in percentage.


  Notice that

      Cov( E , V )
  b = ----------------
        Var(V)

       \Sum_i E[i] V[i]  -  n med(E) med(V)
    = ---------------------------------------------------- =
        \Sum_i V[i]^2  -  n med(V)^2

           SumEV - SumE * SumV / n
    = ----------------------------------
           SumVV - SumV^2 / n


  where
  n = m+1 = number of data,
   
  SumEV := \Sum_i E[i] V[i]
  
  SumVV := \Sum_i V[i]^2
  SumEE : \Sum_i E[i]^2
  SumE := \Sum_i E[i]
  SumV := \Sum_i V[i]    
  the sums are calculated from i= 0,1,..,m


  At the end, the return values are given by
   	SumE := b
    SumV := %R2 or %R2-adjusted (it depends on "ajustado")
  
  where 

  R2 is the determination coefficient (squared r) and
  R2-adjusted is defined by
      1 - [ (n-1)/(n-2) ] ( 1 - r2),
  where n is the number of data.

 
 Reference:
 D. Pea (1987) Estadstica: Modelos y Mtodos: 2. Modelos lineales
  y series temporales. Madrid: Alianza Editorial.
}

  var SumEV, SumVV, SumEE: Tnumero2;
      i: TRgPartido;

Begin
  SumEV:= 0;
  SumVV:= 0;   SumEE:= 0;
  SumE:= 0;    SumV:= 0;

  for i:= 0 to m do
    begin
      SumEV:= SumEV + Pa^[i].Ep * Pa^[i].Vp;
      SumE:= SumE + Pa^[i].Ep;
      SumV:= SumV + Pa^[i].Vp;
      SumVV:= SumVV + SQR(Pa^[i].Vp);
      SumEE:= SumEE + SQR(Pa^[i].Ep);
    end;

  SumEV:= SumEV - SumE * ( SumV/Succ(m) );
  SumVV:= SumVV - SumV * ( SumV/Succ(m) );
  SumEE:= SumEE - SumE * ( SumE/Succ(m) );

  SumE:= SumEV/SumVV;   { SumE <- the estimated slope, b }

  if (SumEE*SumVV) < 1e-15
  then
    SumV:= -1  { r2 is not available = NA }
  else
    begin
      SumV:= SumE*(SumEV/SumEE);   { SQR(SumEV)/(SumEE * SumVV) = r2 }

      if (ajustado and (m>2)) then            
        SumV:= 1.0 - m*(1-SumV)/Pred(m); 
        
      SumV:= SumV*100; { determination coefficient in percentage }
    end;
End;


procedure sesgoEV(Pa: PVecParty;  nodat: TRgPartido;
                  var b, r2: Tnumero);
{ 

It calculates the Cox & Shugart's index of bias in the electoral system (CS),
also known as the regression coefficient.
Further, the (r2%) R-squared associated to CS (expressed in percentage)
is obtained.

The assumed model is given by
 E[i] = a + b V[i] + error[i] , i=0,1,..., nodat     
}

Begin
  modLineal(b, r2, nodat, Pa, FALSE);
End;


function Bus1EscNot0F(var md: TRgPartido; 
                      ndat: TRgPartido;  Pa: PVecParty): boolean;

{ 
It searches for the first party with seats by starting from the end of the sorted sequence
of parties. This way, it is obtained the shortest sequence of most 
voted parties which contains all the parties with parliamentary representation.

The returned value plays a warning role, i.e.,
  Bus1EscNot0F = TRUE means error.

Busca el orden del  primer partido con  escao no nulo,  empezando
por el final. 
}

Begin
  md:= ndat;

  while (Pa^[md].Ep = 0) do
    begin
      if md = 0 then
        begin
          Bus1EscNot0F:= TRUE;
          exit;
        end;

      Dec(md);
    end;

  Bus1EscNot0F:= FALSE;
End;


function Bus1Esc0(var md: TRgPartido;
	          ndat: TRgPartido;  Pa: PVecParty): boolean;

{ 
It searches for the first party with null representation by starting from 
the begining of the sorted sequence of parties. This way it is obtained 
the longest sequence of most voted parties having also 
parliamentary representation.

The return value plays a warning role. Indeed
  Bus1Esc0 = TRUE means error.

Busca el orden del  primer partido con  escao nulo.
}

Begin
  md:= 0;

  while (Pa^[md].Ep > 0) do
    begin
      if md = ndat then
        begin
          Bus1Esc0:= FALSE;
          exit;
        end;

      Inc(md);
    end;

  if md > 0
  then
    begin      
      Dec(md);
      Bus1Esc0:= FALSE;
    end
  else
      Bus1Esc0:= TRUE; { unlikely }
End;


procedure finaliSesgo(var b, r2: Tnumero);
{ 
  It returns the fixed NA (not available) values for the slope and the
determination coefficient.
}
Begin
  b:= 0;
  r2:= -1;
End;


procedure sesgEVp(Pa: PVecParty;  nodat: TRgPartido;
                  var b, r2: Tnumero);

{ 
 The indices CSco and r2% are calculated.
   
 CSco is the modified Cox & Shugart index of bias (CSco)
 It is a version of CS proposed by Oate & Ocaa (1999).
 CSco is computed from the data given by the longest 
 sequence of most voted parties having also parliamentary representation.

 r2% is the R-squared associated to CSco. 

}

  var  md: TRgPartido;

Begin
  if Bus1Esc0(md, nodat, Pa) or (md < 2) then
    begin
      finaliSesgo(b, r2);
      exit;
    end;

  modLineal(b, r2, md, Pa, TRUE);
End;


procedure sesgEVpFin(Pa: PVecParty; nodat: TRgPartido;
                     var b, r2: Tnumero);

{
 The indices CsCF and r2 are calculated.

 CsCF is another modified version of the Cox & Shugart index of bias.
 It is a modified version of CS proposed by Oate & Ocaa (1999).
 CsCF is computed from the data given by the shortest sequence 
 of most voted parties which contains those parties with parliamentary representation.

 r2% is the R-squared associated to CsCF
}


  var md: TRgPartido;

Begin
  if Bus1EscNot0F(md, nodat, Pa) or (md < 2) then
    begin
      finaliSesgo(b, r2);
      exit;
    end;

  modLineal(b, r2, md, Pa,TRUE);
End;



function LineTukey(var b: Tnumero;  m: TRgPartido;
                   Pa: PVecParty): boolean;

{
 Version based on the estimation of quantiles.

 Now some versions of the above indices are obtained by using
 the Tukey line (Exploratory Data Analysis-EDA-), which are 
 proposed in Oate & Ocaa (1999).

 The Tukey line has no sense when m <=2. Because of this,
 the associated value of r2 is used as an auxiliary code.
 If r2 =1, then this means that the Tukey line has sense.

 The return value acts as a warning:
  LineTukey = TRUE means that the Tukey line can not be calculated.
}

  var i1, i3: TRgPartido;

Begin
  i1:= round((m+2)/6);

  if (i1 = 0) or (i1 >= m) then
    begin
      LineTukey:= TRUE;
      exit;
    end;

  Dec(i1);
  i3:= m - i1;

  if i3 <= i1 then
    begin
      LineTukey:= TRUE;
      exit;
    end;

  LineTukey:= FALSE;
  b:= (Pa^[i3].Ep - Pa^[i1].Ep)/(Pa^[i3].Vp - Pa^[i1].Vp);
End;




procedure sesgEVrobu(Pa: PVecParty;  nodat: TRgPartido;
                     var b, r2: Tnumero);

{ the EDA version of CS, denoted by Robu }

Begin
  if LineTukey(b, nodat, Pa) then
    begin
      finaliSesgo(b, r2);
      exit;
    end;

  r2:= 1; 

End;

procedure sesgEVrobuRep(Pa: PVecParty;  nodat: TRgPartido;
                        var b, r2: Tnumero);

{ the EDA version of CSco, denoted by RbRe }

  
  var md: TRgPartido;
      

Begin
  if Bus1Esc0(md, nodat, Pa) 
  then
    finaliSesgo(b, r2)
  else
   begin
    if LineTukey(b, md, Pa) 
    then 
       finaliSesgo(b, r2)
    else 
       r2:= 1;
   end;
End;


procedure sesgEVrobuRepFin(Pa: PVecParty;   nodat: TRgPartido;
                           var b, r2: Tnumero);

{ the EDA version of CScF, denoted by RbRf }

  var md: TRgPartido;

Begin
  if Bus1EscNot0F(md, nodat, Pa) 
  then 
    finaliSesgo(b, r2)
  else
   begin
    if LineTukey(b, md, Pa) 
    then finaliSesgo(b, r2)
    else r2:= 1;
   end;
 
End;


procedure sesgEVrobu05(Pa: PVecParty;  nodat: TRgPartido;
                       var b, r2: Tnumero);

{
 It calculates Rb05, which is th EDA based version 
of  CS from the electoral data given by parties 
with a percentage of votes greather than 0.5%. 
This criterion is inspired on the Lijphart's suggestions
about the Gallagher index of disproportionality.
}

  

  var md: TRgPartido;

Begin
  if BuscaUmbral(md, nodat, 0.5, Pa) 
  then 
    finaliSesgo(b, r2)
  else
    begin
     if LineTukey(b, md, Pa) 
     then finaliSesgo(b, r2)
     else r2:= 1;
    end;
End;


procedure cuadra(var totsumv2, totsume2: Tnumero;
                 Pa: PVecParty;  nodat: TRgPartido);

  var i: TRgPartido;

Begin
  totsumv2:= 0;    totsume2:= 0;

  for i:= 0 to nodat do
    begin
      totsumv2:= totsumv2 + SQR(Pa^[i].Vp * 0.01);
      totsume2:= totsume2 + SQR(Pa^[i].Ep * 0.01);
    end;

End;


function Fragmenta(var totsum2: Tnumero): Tnumero;
{
 Fragmentation  = 1 - \sum_i p_i^2 
}
Begin
  Fragmenta:= 1 - totsum2;
End;


function NoKessWild(votos: boolean;  nodat: TRgPartido;
                    Pa: PVecParty): Tnumero;
{
 the Kesselman & Wildgen's indices of the effective number of parties
}

  var ac, r: Tnumero2;
      i: TRgPartido;

Begin
  ac:= 0;

  for i:= 0 to nodat do
    begin
      if votos then r:= Pa^[i].Vp
               else r:= Pa^[i].Ep;

      if (r > 0)  then
        begin
          r:= r* 0.01;
          ac:= ac + r*Ln(r);
        end; 
    end;

  NoKessWild:= EXP(- ac);
End;


function Molinar(var totsum2: Tnumero;  votos: boolean;
                Pa: PVecParty): Tnumero;
{
  the Molinar's indices of the effective number of parties
}
 var r: Tnumero2;
Begin
  if totsum2 = 0.0 then
    begin
      Molinar:= 0;
      exit;
    end;

  if votos then r:= Pa^[0].Vp
           else r:= Pa^[0].Ep;

  r:= totsum2 - SQR(r * 0.01);

  Molinar:= 1 + r/SQR(totsum2);
End;


procedure NoEfectivo(var gout: Tfichero;
                     Pa: PVecParty;  nodat: TRgPartido;
                     var totsumv2, totsume2: Tnumero);


{
 This procedure calculates all the indices of the effective
number of parties, where

 totsumv2 = \sum_i v[i]^2
 totsume2 = \sum_i e[i]^2

}
 var r1, r2: Tnumero;

 procedure Diferen;
 Begin
   r1:= r1 - r2;
   write(gout, r1);
 End;

Begin
{ The versions of the effective number of parties: 
     electoral and parliamentary 

* The Laakso & Taagepera index }

   r1:= 1/totsumv2;
   write(gout, r1);

   if totsume2 > 0 
   then r2:= 1/totsume2
   else r2:= 0;   { as an outlier }
   write(gout, r2);

   Diferen;

{ * The Kesselman & Wildgen index
 }
   r1:= NoKessWild(TRUE, nodat, Pa);
   write(gout, r1);

   r2:= NoKessWild(FALSE, nodat, Pa);
   write(gout, r2);

   Diferen;

{ * The Molinar index
}
   r1:= Molinar(totsumv2, TRUE, Pa);
   write(gout, r1);

   r2:= Molinar(totsume2, FALSE, Pa);
   write(gout, r2);

   Diferen;
End;


function Concentra(votos: boolean; 
                 Pa: PVecParty; nodat: TRgPartido): Tnumero;

{ Concentration }

  var r: Tnumero;
Begin
  if votos then r:= Pa^[0].Vp
           else r:= Pa^[0].Ep;

  if nodat > 0 then
    begin
      if votos then r:= r + Pa^[1].Vp
               else r:= r + Pa^[1].Ep;
    end;

  Concentra:= r;
End;

function Competitivi(votos: boolean; 
                Pa: PVecParty; nodat: TRgPartido): Tnumero;
{ 
Competitiveness
}
  var r: Tnumero;
Begin
  if votos then r:= Pa^[0].Vp
           else r:= Pa^[0].Ep;

  if nodat > 0 then
    begin
      if votos  then  r:= r - Pa^[1].Vp
                else  r:= r - Pa^[1].Ep;
      { The parlamentary version can return negative values.
        En el caso parlamentario puede salir negativo }
    end;

  Competitivi:=  r;
End;


function iniPolariz(var camino: TcaminFi;
                   Pa: PVecParty; pP: PVector; nodat: TRgPartido): boolean;

  var i: TRgPartido;
      fpola: text;

Begin
  for i:= 0 to nodat do pP^[i] := -2; {an auxiliary value (NA label)}

  {$I-}
  assign(fpola, camino+'siglapo.txt');  reset(fpola);
  {$I+}

  if IoResult <> 0 then
    begin
      iniPolariz:= FALSE;
      exit;
    end;

  for i:= 0 to nodat do
      tomaPolariza(Pa^[i].SiglasCandi, fpola, pP^[i]);
  
  close(fpola);
  iniPolariz:= TRUE;
End;


function PolaSartori(pP: PVector; nodat: TRgPartido): Tnumero;

  var mini, maxi: Tnumero;
      i: TRgPartido;
      
Begin
  maxi:= -1;     mini:= 11;  

  for i:= 0 to nodat do
    if (pP^[i] > -1)  then
      begin        
        if pP^[i] < mini  then  mini:= pP^[i];
        if maxi < pP^[i]  then  maxi:= pP^[i];
      end;

  if maxi < 0  then PolaSartori:= 0
               else PolaSartori:= maxi - mini;

End;


function PolaPondera(votos: boolean; Pa: PVecParty; pP: PVector; 
                  nodat: TRgPartido): Tnumero;


  var media, PesoTotal, peso, ac: Tnumero;
      cont, i: TRgPartido;
Begin
  media:= 0;
  cont:= 0;
  PesoTotal:= 0;

  for i:= 0 to nodat do
    if pP^[i] > -1 then
      begin
        if votos then peso:= Pa^[i].Vp
                 else peso:= Pa^[i].Ep;
        peso:= peso*0.01;

        Inc(cont);
        media:= media + peso*pP^[i];
        PesoTotal:= PesoTotal + peso;
      end;


  if (cont = 0) or (PesoTotal < 1e-5) then
    begin
      PolaPondera:= 0;
      exit;
    end;

  media:= media/PesoTotal;
  ac:= 0;

  for i:= 0 to nodat do
   if pP^[i] > -1 then
     begin
       if votos then peso:= Pa^[i].Vp
                else peso:= Pa^[i].Ep;

       peso:= peso * 0.01;
       ac:= ac + peso * SQR(pP^[i] - media);
     end;

  PolaPondera:= ac/PesoTotal;
End;


procedure Medidas(var camino: TcaminFi;  var gout: Tfichero; 
                  Pa: PVecParty;  nodat: TRgPartido;
                  var totv, tote: Tnumero);

{
 This procedure stores in a binary file a lot of indices,
 where
  totv = \sum_i V[i]
  tote = \sum_i E[i]
}


  var b, rx: Tnumero;
      ip: TRgPartido;
      pD: PVectoR;

Begin
{     **************************
       Disproportionality indices
       **************************
}
  iniDisproportion(Pa, pD, nodat);

 { 1. the Sainte-Lague index   }
  rx:= Chi2Saint(Pa, pD, nodat);
  write(gout, rx);

 { 2. the Rae index }
  rx:= DsprRae(pD, nodat);
  write(gout, rx);

 { 3. the modified Rae index }
  rx:= DsprRaeCo(Pa, pD, nodat);
  write(gout, rx);

 { 4. the Loosemore & Hanby index  }
  rx:= DsprLooHan(pD, nodat);
  write(gout, rx);

 { 5. the Mackie & Rose index   }
  rx:= 100 - rx;
  write(gout, rx);

 { 6. the Grofman index of electoral disproportionality }
  rx:= DsprGrofman(TRUE, Pa, pD, nodat);
  write(gout, rx);

 { 7. the Grofman index of parliamentary disproportionality }
  rx:= DsprGrofman(FALSE, Pa, pD, nodat);
  write(gout, rx);


 { 8. the Lijphart index of maximum deviation
   9. the index of party having maximum deviation
 }
  DsprLijMaxD(pD, nodat, rx, ip);
  write(gout, rx);
  rx:= ip;    write(gout, rx);

 {10. the Gallagher index }
  rx:= DsprMCGa(pD, nodat);
  write(gout, rx);

 {11. the Lijphart index }
  rx:= DsprMCLij(Pa, pD, nodat);
  write(gout, rx);

{ 12. the robust L1-norm based index [Pennisi, 1998]  }
  rx:= DsprRobuL1(Pa, nodat, totv, tote);
  write(gout, rx);

 { 13. the robust L2-norm based index [Pennisi, 1998] }
  rx:= DsprRobuL2(Pa, nodat, totv, tote);
  write(gout, rx);

 { 14. the robust Loo-norm based index [Pennisi, 1998]
   15. the index of party achieving the maximum norm  }
  DsprRobuLoo(Pa, nodat, ip, totv, tote, rx);
  write(gout, rx);
  rx:= ip;    write(gout, rx);

 { 16. the Cox & Shugart index of bias in the electoral system
   17. Coefficient r2 % }
  sesgoEV(Pa, nodat, b, rx);
  write(gout, b, rx);

 { 18. the corrected Cox & Shugart index
   19. Coefficient r2 %  (Distinguidor negativo ) }
  sesgEVp(Pa, nodat, b, rx);
  write(gout, b, rx);


 { 20. the end-corrected Cox & Shugart index
   21. Coefficient r2 % (Distinguidor negativo) }
  sesgEVpFin(Pa, nodat, b, rx);
  write(gout, b, rx);

 { 22. the robust bias index - Indice de sesgo robusto
   23. Coefficient discriminator }
  sesgEVrobu(Pa, nodat, b, rx);
  write(gout, b, rx);

 { 24. the corrected robust bias index - Indice de sesgo robusto-rep
   25. Coefficient discriminator }
  sesgEVrobuRep(Pa, nodat, b, rx);
  write(gout, b, rx);

 { 26. the end-corrected robust bias index- Indice sesgo robusto-rep-fin
   27. Coefficient discriminator }
  sesgEVrobuRepFin(Pa, nodat, b, rx);
  write(gout, b, rx);

{  28. the 0.5%-corrected bias index - Indice sesgo robusto-0.5%
   29. Coefficient discriminator }
  sesgEVrobu05(Pa, nodat, b, rx);
  write(gout, b, rx);

{ PARTY SYSTEM DIMENSIONS }

  cuadra(totv, tote, Pa, nodat);
{   
    totv = \sum_i V[i]^2
    tote = \sum_i E[i]^2   

   30. Electoral Fragmentation
   31. Parliamentary Fragmentation }
  rx:= Fragmenta(totv);        write(gout, rx);
  rx:= Fragmenta(tote);        write(gout, rx);

{    32. Electoral Concentration
     33. Parliamentary Concentration }
  rx:= Concentra(TRUE, Pa, nodat);      write(gout, rx);
  rx:= Concentra(FALSE, Pa, nodat);     write(gout, rx);

{   34. Electoral Competitiveness
    35. Parliamentary Competitiveness }
  rx:= Competitivi(TRUE, Pa, nodat);    write(gout, rx);
  rx:= Competitivi(FALSE, Pa, nodat);   write(gout, rx);


{ N = the Laakso & Taagepera indices of effective number of parties
    36. the electoral version of N
    37. the parliamentary version of N
    38. Difer. No. efect. Laakso & Taagepera: electoral - parlamentario
  
  I = the Kesselman & Wildgen index of effective number of parties
    39. the electoral version of I
    40. the parliamentary version of I 
    41. Difer. No. efect. Kesselman & Wildgen: elect - parlam.

  NP = the Molinar index of effective number of parties
    42. the electoral version of NP
    43. the parliamentary version of NP
    44. Difer. No. efectiv. Molinar: elect - parlament 
}
  NoEfectivo(gout, Pa, nodat, totv, tote);

{ the indices of polarization  
    45. the Sartori index
    46. the electoral weighted index (by Hazan)
    47. the parliamentary weighted index (by Hazan)
    48. Diference: PHz_e - PHz_p 
}

  if iniPolariz(camino, Pa, pD, nodat)
  then
    begin
      rx:= PolaSartori(pD, nodat);
      write(gout, rx);

      rx:= PolaPondera(TRUE, Pa, pD, nodat);
      write(gout, rx);

      b:= PolaPondera(FALSE, Pa, pD, nodat);
      write(gout, b);

      rx:= rx - b;
      write(gout, rx);
    end
  else
    begin
      rx:= 0;
      for ip:= 0 to 3 do write(gout, rx);
    end;

  DelVectoR(pD, nodat);
End;

{ ********************************************************************* }

procedure IteraMC(var f: text;  Pa: PVecParty; pD: PVector; ndat: TRgPartido);

{ Sequence of least-squares indices of disproportionality % =
    SQRT( 0.5 *  \sum_i ( V[i] - E[i] )^2  ) 
 }
  const tabcolu = 10;

  var ac, Indice: Tnumero2;
      i, ip: TRgPartido;

Begin
  BuscaUmbral(ip, ndat, 0.5, Pa);

  writeln(f,' Sequence of scores of the LS index of disproportionality (%)');
  writeln(f,'   ( * denotes the 0.5% threshold )');

  write(f,'m ':4);   write(f, charTabu);
  write(f, '  ');
  write(f,'Index (m) ':tabcolu);  writeln(f, charTabu);

  ac:= 0;

  for i:= 0 to ndat do
    begin
      write(f, ' ');
      write(f, Succ(i):2 );
      write(f, ' '); write(f, charTabu);

      ac := ac + SQR(pD^[i]);
      Indice:= SQRT( ac * 0.5);

      if i = ip
      then
        begin
          escridat(f, Indice,tabcolu,preci, TRUE, FALSE);
          writeln(f, ' * ');
        end
      else
          escridat(f, Indice, tabcolu,preci,TRUE, TRUE);

    end;

  writeln(f);
End;


procedure IteraRae(var f: text; Pa: PVecParty; pD: PVector; ndat: TRgPartido);
{ 
Sequence of partial averaged deviations % = 
      \sum_i | V[i] - E[i] | / n
}

 const tabcolu = 10;

  var
      i, ip: TRgPartido;
      ac, Indice: Tnumero2;

Begin
  BuscaUmbral(ip, ndat, 0.5, Pa);

  writeln(f, ' Sequence of scores of the Rae index (%) ');
  writeln(f, '   ( * denotes the 0.5% threshold ) ');

  write(f,'m ':4);   write(f, charTabu);
  write(f, '  ');
  write(f,'Index (m) ':tabcolu);   writeln(f, charTabu);

  ac:= 0;

  for i:= 0 to ndat do
    begin
      write(f, ' ');
      write(f, Succ(i):2 );
      write(f, ' ');  write(f, charTabu);

      ac := ac + ABS(pD^[i]);
      Indice:= ac/ Succ(i);

      if i = ip
      then
        begin
          escridat(f, Indice, tabcolu, preci, TRUE, FALSE);
          writeln(f, ' * ');
        end
      else
          escridat(f, Indice, tabcolu, preci, TRUE, TRUE);

    end;

  writeln(f);
End;



procedure NumeroPartidoTXT(var f:text;  
                        Pa: PVecParty; nodat: TRgPartido;
                        var totv, tote: Tnumero);

 const
     tabuln = 20;
     tabula = 10;
     precisip = 2;

 var r1, r2: Tnumero;

 procedure Diferen;
 Begin
   r1:= r1 - r2;
   escridat(f, r1,tabula,precisip,TRUE, TRUE);
 End;

Begin
  writeln(f);
  writeln(f, ' Effective number of parties : ');
  writeln(f);

  write(f, 'Indices  ':tabuln);      write(f, charTabu);
  write(f, 'Electoral':tabula);      write(f, charTabu);
  write(f, 'Parliame.':tabula);      write(f, charTabu);
  write(f, 'Diferen.':tabula);       writeln(f, charTabu);

  write(f,'Laakso & Taagepera':tabuln); write(f, charTabu);

   r1:= 1/totv;
   escridat(f, r1,tabula, precisip, TRUE, FALSE);

   r2:= 1/tote;
   escridat(f, r2,tabula,precisip,TRUE, FALSE);

   Diferen;


  write(f, 'Kesselman & Wildgen':tabuln);   write(f, charTabu);

   r1:= NoKessWild(TRUE, nodat, Pa);
   escridat(f, r1,tabula,precisip,TRUE,FALSE);

   r2:= NoKessWild(FALSE, nodat, Pa);
   escridat(f, r2,tabula,precisip,TRUE,FALSE);

   Diferen;

  write(f, 'Molinar':tabuln);    write(f, charTabu);

   r1:= Molinar(totv, TRUE, Pa);
   escridat(f, r1, tabula, precisip, TRUE, FALSE);

   r2:= Molinar(tote, FALSE, Pa);
   escridat(f, r2, tabula, precisip, TRUE, FALSE);

   Diferen;

  writeln(f);
  writeln(f);
End;


procedure NumeroPartidHTML(var f:text; 
                        Pa: PVecParty;  nodat: TRgPartido;
                        var totv, tote: Tnumero);

{ HTML version of effective number of parties }

 const  precisip = 2;

 var r1, r2: Tnumero;

 procedure Diferen1;
 Begin
   r1:= r1 - r2;
   escridaHT(f, r1, precisip);
 End;

Begin
  writeln(f, '<h3>');
  write(f, ' Effective number of parties : ');
  writeln(f, '</h3>');

  writeln(f, '<p> <table cellspacing="8"> <tr>');
  writeHeadTabla(f, 'Indices  ', 'l');
  writeHeadTabla(f, 'Electoral', 'r');
  writeHeadTabla(f, 'Parliame.', 'r');
  writeHeadTabla(f, 'Diferen.', 'r');
  writeln(f, '</tr> <tr>');

  writeHeadTabla(f,'Laakso & Taagepera', 'l');

   r1:= 1/totv;
   escridaHT(f, r1, precisip);

   r2:= 1/tote;
   escridaHT(f, r2, precisip);

   Diferen1;

  writeln(f, '</tr> <tr>');
  writeHeadTabla(f, 'Kesselman & Wildgen', 'l');

   r1:= NoKessWild(TRUE, nodat, Pa);
   escridaHT(f, r1, precisip);

   r2:= NoKessWild(FALSE, nodat, Pa);
   escridaHT(f, r2, precisip);

   Diferen1;

  writeln(f, '</tr> <tr>');
  writeHeadTabla(f, 'Molinar', 'l');

   r1:= Molinar(totv, TRUE, Pa);
   escridaHT(f, r1, precisip);

   r2:= Molinar(tote, FALSE, Pa);
   escridaHT(f, r2, precisip);

   Diferen1;

  writeln(f, '</tr> </table> </p>');
  flush(f);
End;


procedure NumeroPartidoRS(var f:text;
                        Pa: PVecParty; nodat: TRgPartido;
                        var totv, tote: Tnumero);

 const
     tabuln = 20;
     tabula = 10;
     precisip = 2;

 var r1: Tnumero;


Begin
  writeln(f);
  writeln(f, '# Effective number of parties');
  writeln(f);

  write(f, 'EfNumberParties <- list(');

  r1:= 1/totv;
  write(f,'LaaksoTaagepera.e = ');
  escridatLIST(f, r1, precisip, TRUE, FALSE);

  r1:= 1/tote;
  write(f,'LaaksoTaagepera.p = ');
  escridatLIST(f, r1, precisip, TRUE, FALSE);

  r1:= NoKessWild(TRUE, nodat, Pa);
  write(f, 'KesselmanWildgen.e = ');
  escridatLIST(f, r1, precisip, TRUE, FALSE);

  r1:= NoKessWild(FALSE, nodat, Pa);
  write(f, 'KesselmanWildgen.p = ');
  escridatLIST(f, r1, precisip, TRUE, FALSE);

  r1:= Molinar(totv, TRUE, Pa);
  write(f, 'Molinar.e = ');
  escridatLIST(f, r1, precisip, TRUE, FALSE);

  r1:= Molinar(tote, FALSE, Pa);
  write(f, 'Molinar.p = ');
  escridatLIST(f, r1, precisip, FALSE, TRUE);

  writeln(f, ');');
  writeln(f);
  flush(f);
End;



procedure DesproDiagra(var f: text; Pa: PVecParty; pD: PVector; ndat: TRgPartido);

label venga;

const tabcol = 5;
      len_plot = 21;  { it must be an odd number }


var cadde: string[len_plot];
    i: TRgPartido;
    dej: integer;
    j, posi: byte;

Begin
  writeln(f);
  writeln(f, 'PLOT OF DEVIATIONS %Seats - %Votes':55);
  writeln(f);

  write(f, 'No':4);                   write(f, charTabu);
  write(f, 'Party':lenCandida);       write(f, charTabu);
  write(f, ' ':len_plot);             write(f, charTabu);
  write(f, '%Deviation':tabcol);      writeln(f, charTabu);

  for i:= 0 to ndat do
    begin
      write(f, ' ');
      write(f, Succ(i):3);   write(f, charTabu);

      write(f, Pa^[i].SiglasCandi: lenCandida);  write(f, charTabu);

      dej:= round(pD^[i]);

      if ABS(dej) > (len_plot shr 1) then
        begin
          dej:= (len_plot shr 1);
          if pD^[i] < 0 then dej:= - dej;
        end;

      FillChar(cadde, Succ(len_plot), ' ');
      cadde[0]:= chr(len_plot);
      cadde[ Succ(len_plot) shr 1]:= '|';

      if dej = 0 then goto venga;

      if dej < 0 then posi:= (len_plot shr 1) + dej
                 else posi:= Succ(len_plot) shr 1;

      for j:=1 to ABS(dej) do cadde[posi + j] := '*';

      venga:
      write(f, cadde);    write(f, charTabu);

      escridat(f, pD^[i], tabcol, 2, TRUE, TRUE);
    end;

  writeln(f);

  flush(f);

End;


procedure MedidasTXT(var camino: TcaminFi;  var f: text; 
                     Pa: PVecParty; nodat: TRgPartido;
                     var totv, tote: Tnumero);

  var b, rx: Tnumero;
      ip: TRgPartido;
      pD: PVector;

Begin
  iniDisproportion(Pa, pD, nodat);

  DesproDiagra(f, Pa, pD, nodat);

  writeln(f);
  writeln(f, ' DISPROPORTIONALITY OF THE ELECTORAL SYSTEM');
  writeln(f);

  rx:= Chi2Saint(Pa,pD, nodat);
  write(f, ' Sainte-Lague index (%) = ');
  escridat(f, rx, 2, preci, FALSE, TRUE);
  writeln(f);

  rx:= DsprRae(pD, nodat);
  write(f,' Rae index (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f);

  rx:= DsprRaeCo(Pa, pD, nodat);
  write(f, ' Corrected Rae index (%) = ');
  escridat(f, rx, 2, preci, FALSE, TRUE);
  writeln(f);

  IteraRae(f, Pa, pD, nodat);

  rx:= DsprLooHan(pD, nodat);
  write(f,' Loosemore & Hanby index (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f);

  rx:= 100 - rx;
  write(f, ' (*) Mackie & Rose index of proportionality (%) = ');
  escridat(f, rx, 2, preci, FALSE, TRUE);
  writeln(f);

  rx:= DsprGrofman(TRUE, Pa, pD, nodat);
  write(f,' Electoral Grofman index (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f);

  rx:= DsprGrofman(FALSE, Pa, pD, nodat);
  write(f,' Parliamentary Grofman index (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f);

  DsprLijMaxD(pD, nodat, rx, ip);
  write(f,' Maximum deviation based Lijphart index (%) = ');
  escridat(f, rx, 2, preci, FALSE, FALSE);
  write(f, ' (Party = ');
  write(f, ip);
  writeln(f, ')');
  writeln(f);

  rx:= DsprMCGa(pD, nodat);
  write(f,' Least squares Gallagher index (%) = ');
  escridat(f, rx,2,preci,FALSE,TRUE);
  writeln(f);

  rx:= DsprMCLij(Pa, pD, nodat);
  write(f,' Least squares index corrected by Lijphart (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f);

  IteraMC(f, Pa, pD, nodat);


  sesgoEV(Pa, nodat, b, rx);
  write(f,' Cox & Shugart index of bias = ');
  escridat(f, b,2,preci, FALSE, FALSE);
  write(f,' ':3);
  write(f,'( r^2 (%) = ');
  escridat(f, rx, 2, preci, FALSE, FALSE);
  writeln(f, ' )');
  writeln(f);

  sesgEVp(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,' Corrected index of bias = ');
      escridat(f, b,2,preci,FALSE, FALSE);
      write(f,' ':3);
      write(f,'( r^2 (%) = ');
      escridat(f, rx, 2, preci, FALSE, FALSE);
      writeln(f, ' )');
      writeln(f);
    end;

  sesgEVpFin(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,' Corrected-End index of bias = ');
      escridat(f, b,2,preci,FALSE, FALSE);
      write(f,' ':3);
      write(f,'( r^2 (%) = ');
      escridat(f, rx, 2, preci, FALSE, FALSE);
      writeln(f, ' )');
      writeln(f);
    end;

  sesgEVrobu(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,' Robust index of bias = ');
      escridat(f, b,2,preci, FALSE, TRUE);
      writeln(f);
    end;

  sesgEVrobuRep(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,' Robust-Rep index of bias = ');
      escridat(f, b,2,preci,FALSE,TRUE);
      writeln(f);
    end;

  sesgEVrobuRepFin(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,' Robust-Rep-End index of bias = ');
      escridat(f, b,2,preci,FALSE,TRUE);
      writeln(f);
    end;

  sesgEVrobu05(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,' Robust-0.5% index of bias = ');
      escridat(f, b,2,preci,FALSE,TRUE);
      writeln(f);
    end;


  writeln(f);
  writeln(f, 'DIMENSIONS OF THE PARTY SYSTEM':50);
  writeln(f);


  cuadra(totv, tote, Pa, nodat);
{   totv <- suma cuadrados vi
    tote <- suma cuadrados ei }

  rx:= Fragmenta(totv);
  write(f,' Electoral fragmentation = ');
  escridat(f, rx, 2, preci, FALSE, TRUE);
  writeln(f);

  rx:= Fragmenta(tote);
  write(f,' Parliamentary fragmentation = ');
  escridat(f, rx, 2, preci, FALSE, TRUE);
  writeln(f);

  rx:= Concentra(TRUE, Pa, nodat);
  write(f, ' Electoral concentration (%)  = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f);

  rx:= Concentra(FALSE, Pa, nodat);
  write(f, ' Parliamentary concentration (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f);

  rx:= Competitivi(TRUE, Pa, nodat);
  write(f, ' Electoral competitiveness (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f);

  rx:= Competitivi(FALSE, Pa, nodat);
  write(f, ' Parliamentary competitiveness (%) = ');
  escridat(f, rx,2,preci,FALSE, TRUE);
  writeln(f);

  NumeroPartidoTXT(f, Pa, nodat, totv, tote);

  { 45. Sartori polarization index
    46. electoral Weighted polarization [Hazan]
    47. parliamentary weighted polarization [Hazan]
    48. Difference: PHz_e - PHz_p }


  if iniPolariz(camino, Pa, pD, nodat)   then
    begin
      writeln(f);
      writeln(f, 'POLARIZATION':30);
      writeln(f);

      rx:= PolaSartori(pD, nodat);
      write(f, ' Sartori Index = ');
      escridat(f, rx, 2, preci, FALSE, TRUE);
      writeln(f);
      writeln(f);

      writeln(f, ' Weighted polatization indices (PP) : ');
      rx:= PolaPondera(TRUE, Pa, pD, nodat);
      write(f, '  * Electoral weighted polarization = ');
      escridat(f, rx, 2, preci, FALSE, TRUE);
      writeln(f);

      b:= PolaPondera(FALSE, Pa, pD, nodat);
      write(f, '  * Parliamentary weighted polarization = ': 12);
      escridat(f, b, 2, preci, FALSE, TRUE);
      writeln(f);

      rx:= rx - b;
      write(f, ' Diference between weighted polarization indices = ');
      escridat(f, rx, 2, preci, FALSE, TRUE);
      writeln(f);
    end;

  DelVectoR(pD, nodat);
End;

{ ********************************************************************* }


procedure FragConcCompeHTML(var f: text; Pa: PVecParty; nodat: TRgPartido;
                            var totv, tote: Tnumero);

{   totv <- suma cuadrados vi
    tote <- suma cuadrados ei 
}

var rx: Tnumero;
Begin
  writeln(f,'<h3>Fragmentation, Concentration and Competitiveness (%)</h3>');

  writeln(f, '<p><table>');
  write(f, '<tr><td> </td>');
  writeHeadTabla(f, 'Electoral', 'r');
  writeHeadTabla(f, 'Parliame.', 'r');
  writeln(f,'</tr>');

  writeln(f, '<tr><th>Fragmentation</th>');
{  Electoral  fragmentation }
  rx:= Fragmenta(totv);
  escridaHT(f, rx, preci);

{  Parliamentary fragmentation }
  rx:= Fragmenta(tote);
  escridaHT(f, rx, preci);

  writeln(f, '</tr><tr><th>Concentration</th>');
{   Electoral concentration }
  rx:= Concentra(TRUE, Pa, nodat);
  escridaHT(f, rx, preci);

{   Parliamentary concentration }
  rx:= Concentra(FALSE, Pa, nodat);
  escridaHT(f, rx, preci);

  writeln(f, '</tr><tr><th>Competitiveness</th>');
{ Electoral competitiveness }
  rx:= Competitivi(TRUE, Pa, nodat);
  escridaHT(f, rx, preci);

{ Parliamentary competitiveness }
  rx:= Competitivi(FALSE, Pa, nodat);
  escridaHT(f, rx, preci);

  writeln(f, '</tr></table></p>');
  flush(f);
End;


procedure MedidasHTML(var camino: TcaminFi;  var f: text; 
                     Pa: PVecParty;  nodat: TRgPartido;
                     var totv, tote: Tnumero);

  var b, rx: Tnumero;
      ip: TRgPartido;
      pD: PVectoR;

Begin
  iniDisproportion(Pa, pD, nodat);

  writeln(f, '<p><pre>');
  DesproDiagra(f, Pa, pD, nodat);
  writeln(f, '</pre></p>');

  writeln(f, '<h2>');
  write(f, 'Disproportionality of the electoral system');
  writeln(f, '</h2>');

  write(f, '<p> <ul>');

  rx:= Chi2Saint(Pa, pD, nodat);
  write(f, '<li>Saint-Lague index (%) = ');
  escridat(f, rx, 2, preci, FALSE, TRUE);
  writeln(f, '</li>');

  rx:= DsprRae(pD, nodat);
  write(f,'<li>Rae index (%) = ');
  escridat(f, rx, 2, preci, FALSE, TRUE);
  writeln(f, '</li>');

  rx:= DsprRaeCo(Pa,pD, nodat);
  write(f, '<li>Corrected Rae index (%) = ');
  escridat(f, rx, 2, preci, FALSE, TRUE);
  writeln(f, '</li>');
  writeln(f, '</ul></p>');

  writeln(f, '<p><pre>');
  IteraRae(f, Pa, pD, nodat);
  writeln(f, '</pre></p>');

  writeln(f, '<p><ul>');
  rx:= DsprLooHan(pD, nodat);
  write(f,'<li>Loosemore & Hanby index (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f, '</li>');

  rx:= 100 - rx;
  write(f, '<li>Mackie & Rose index of proportionality (%) = ');
  escridat(f, rx, 2, preci, FALSE, TRUE);
  writeln(f, '</li>');

  rx:= DsprGrofman(TRUE, Pa, pD,nodat);
  write(f,'<li>Electoral Grofman index (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f, '</li>');

  rx:= DsprGrofman(FALSE, Pa, pD,nodat);
  write(f,'<li>Parliamentary Grofman index (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f, '</li>');

  DsprLijMaxD(pD, nodat, rx, ip);
  write(f,'<li>Maximum deviation based Lijphart index (%) = ');
  escridat(f, rx, 2, preci, FALSE, FALSE);
  write(f, ' -- (Party = ');
  write(f, ip);
  writeln(f, ')');
  writeln(f, '</li>');

  rx:= DsprMCGa(pD, nodat);
  write(f,'<li>Least squares Gallagher index (%) = ');
  escridat(f, rx,2,preci,FALSE,TRUE);
  writeln(f, '</li>');

  rx:= DsprMCLij(Pa, pD,nodat);
  write(f,'<li>Least squares index corrected by Lijphart (%) = ');
  escridat(f, rx,2,preci, FALSE, TRUE);
  writeln(f, '</li>');
  writeln(f, '</ul></p>');

  writeln(f, '<p><pre>');
  IteraMC(f, Pa, pD, nodat);
  writeln(f, '</pre></p>');

  writeln(f, '<p><ul>');

  sesgoEV(Pa, nodat, b, rx);
  write(f,'<li>Cox & Shugart index of bias = ');
  escridat(f, b,2,preci, FALSE, FALSE);
  write(f,' -- ( r<sup>2</sup> (%) = ');
  escridat(f, rx, 2, preci, FALSE, FALSE);
  writeln(f, ' )</li>');

  sesgEVp(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'<li>Corrected index of bias = ');
      escridat(f, b,2,preci,FALSE, FALSE);
      write(f,' -- ( r<sup>2</sup> (%) = ');
      escridat(f, rx, 2, preci, FALSE, FALSE);
      writeln(f, ' )');
      writeln(f, '</li>');
    end;

  sesgEVpFin(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'<li>Corrected-End index of bias = ');
      escridat(f, b,2,preci,FALSE, FALSE);
      write(f,' ':3);
      write(f,'( r<sup>2</sup> (%) = ');
      escridat(f, rx, 2, preci, FALSE, FALSE);
      writeln(f, ' )');
      writeln(f, '</li>');
    end;

  sesgEVrobu(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'<li>Robust index of bias = ');
      escridat(f, b,2,preci, FALSE, TRUE);
      writeln(f, '</li>');
    end;

  sesgEVrobuRep(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'<li>Robust-Rep index of bias = ');
      escridat(f, b,2,preci,FALSE,TRUE);
      writeln(f, '</li>');
    end;

  sesgEVrobuRepFin(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'<li>Robust-Rep-End index of bias = ');
      escridat(f, b,2,preci,FALSE,TRUE);
      writeln(f, '</li>');
    end;

  sesgEVrobu05(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'<li>Robust-0.5% index of bias = ');
      escridat(f, b,2,preci,FALSE,TRUE);
      writeln(f, '</li>');
    end;


  writeln(f, '</ul> </p>  <hr> <p> </p> <h2>');
  writeln(f, 'Dimensions of party system');
  writeln(f, '</h2>');


  cuadra(totv, tote, Pa, nodat);
{   totv <- suma cuadrados vi
    tote <- suma cuadrados ei }

  FragConcCompeHTML(f, Pa, nodat, totv, tote);

  NumeroPartidHTML(f, Pa, nodat, totv, tote);

 {  45. Sartori polarization index
    46. electoral Weighted polarization [Hazan]
    47. parliamentary weighted polarization [Hazan]
    48. Difference: PHz_e - PHz_p }


  if iniPolariz(camino, Pa, pD,nodat)   then
    begin
      writeln(f, '<h2>');
      write(f, 'Indices of polarization');
      writeln(f, '</h2>  <p> <ul> ');

      rx:= PolaSartori(pD, nodat);
      write(f, '<li><p>Sartori Index = ');
      escridat(f, rx, 2, preci, FALSE, TRUE);
      writeln(f, '</p></li>');

      writeln(f, '<li><p>Hazan weighted indices (PP) : <ul>');
      rx:= PolaPondera(TRUE, Pa, pD,nodat);
      write(f, '<li>Electoral weighted polarization = ');
      escridat(f, rx, 2, preci, FALSE, TRUE);
      writeln(f, '</li>');

      b:= PolaPondera(FALSE, Pa, pD,nodat);
      write(f, '<li>Parliamentary weighted polarization = ');
      escridat(f, b, 2, preci, FALSE, TRUE);
      writeln(f, '</li>');

      rx:= rx - b;
      write(f, '<li>Diference between weighted indices = ');
      escridat(f, rx, 2, preci, FALSE, TRUE);
      writeln(f, '</li></ul> </p></li> </ul></p>');
    end;

  DelVectoR(pD, nodat);

  writeln(f, '</body>');
  writeln(f, '</html>');
End;

{ ********************************************************************* }



procedure MedidasRS(var camino: TcaminFi;  var f: text;
                     Pa: PVecParty; nodat: TRgPartido;
                     var totv, tote: Tnumero);

  var b, rx: Tnumero;
      ip: TRgPartido;
      pD: PVectoR;

Begin
  iniDisproportion(Pa, pD, nodat);

  writeln(f);
  writeln(f, '# DISPROPORTIONALITY OF THE ELECTORAL SYSTEM');

  writeln(f);
  write(f, 'Disproportion <- list(');

  rx:= Chi2Saint(Pa,pD, nodat);
  write(f, 'SainteLague = ');
  escridatLIST(f, rx, preci, TRUE, FALSE);

  rx:= DsprRae(pD, nodat);
  write(f,'Rae = ');
  escridatLIST(f, rx, preci, TRUE, FALSE);

  rx:= DsprRaeCo(Pa, pD, nodat);
  write(f, 'Rae.corrected = ');
  escridatLIST(f, rx, preci, TRUE, FALSE);

  rx:= DsprLooHan(pD, nodat);
  write(f,'LoosemoreHanby = ');
  escridatLIST(f, rx, preci, TRUE, FALSE);

  rx:= 100 - rx;
  write(f, 'MackieRose = ');
  escridatLIST(f, rx, preci, TRUE, FALSE);

  rx:= DsprGrofman(TRUE, Pa,pD, nodat);
  write(f,'Grofman.electo = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);

  rx:= DsprGrofman(FALSE, Pa,pD, nodat);
  write(f,'Grofman.parliam = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);

  DsprLijMaxD(pD, nodat, rx, ip);
  write(f,'Lijphat.max.index = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);
  write(f, 'Lijphat.max.iparty = ');
  write(f, ip);
  writeln(f, ',');

  rx:= DsprMCGa(pD, nodat);
  write(f,'Gallagher = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);

  rx:= DsprMCLij(Pa,pD, nodat);
  write(f,'Lijphart.LS = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);

  sesgoEV(Pa, nodat, b, rx);
  write(f,'CoxShugart.slope = ');
  escridatLIST(f, b,preci, TRUE, FALSE);
  write(f,'CoxShugart.R2 = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);

  sesgEVp(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'Corrected.index.bias = ');
      escridatLIST(f, b,preci, TRUE, FALSE);
      write(f,'Corrected.index.R2 = ');
      escridatLIST(f, rx,preci, TRUE, FALSE);
    end;

  sesgEVpFin(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'CorrectedEnd.index.bias = ');
      escridatLIST(f, b,preci, TRUE, FALSE);
      write(f,'CorrectedEnd.index.R2 = ');
      escridatLIST(f, rx,preci, TRUE, FALSE);
    end;

  sesgEVrobu(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'Robust.index.bias = ');
      escridatLIST(f, b,preci, TRUE, FALSE);
    end;

  sesgEVrobuRep(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'RobustRep.index.bias = ');
      escridatLIST(f, b,preci, TRUE, FALSE);
    end;

  sesgEVrobuRepFin(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'RobustRepEnd.index.bias = ');
      escridatLIST(f, b,preci, TRUE, FALSE);
    end;

  sesgEVrobu05(Pa, nodat, b, rx);
  if rx > 0 then
    begin
      write(f,'Robust05.index.bias = ');
      escridatLIST(f, b,preci, FALSE, TRUE);
    end;

  writeln(f, ');');

  writeln(f);
  writeln(f, '#  DIMENSIONS OF PARTY SYSTEMS');  
  writeln(f);

  write(f, 'SomeDimens <- list(');

  cuadra(totv, tote, Pa, nodat);
{   totv <- suma cuadrados vi
    tote <- suma cuadrados ei }

  rx:= Fragmenta(totv);
  write(f,'Fragmentation.e = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);

  rx:= Fragmenta(tote);
  write(f,'Fragmentation.p = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);

  rx:= Concentra(TRUE, Pa, nodat);
  write(f, 'Concentration.e = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);

  rx:= Concentra(FALSE, Pa, nodat);
  write(f, 'Concentration.p = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);

  rx:= Competitivi(TRUE, Pa, nodat);
  write(f, 'Competitiveness.e = ');
  escridatLIST(f, rx,preci, TRUE, FALSE);

  rx:= Competitivi(FALSE, Pa, nodat);
  write(f, 'Competitiveness.p = ');
  escridatLIST(f, rx,preci, FALSE, TRUE);
  writeln(f, ');');

  NumeroPartidoRS(f, Pa, nodat, totv, tote);

  { 45. Sartori polarization index
    46. electoral Weighted polarization [Hazan]
    47. parliamentary weighted polarization [Hazan]
    48. Difference: PHz_e - PHz_p }


  if iniPolariz(camino, Pa, pD, nodat)   then
    begin
      writeln(f);
      writeln(f, '# Polarization');
      writeln(f);

      write(f, 'Polarization <- list(' );

      rx:= PolaSartori(pD, nodat);
      write(f, 'Sartori = ');
      escridatLIST(f, rx,preci, TRUE, FALSE);

      writeln(f, '# Weighted polatization indices (PP) : ');
      rx:= PolaPondera(TRUE, Pa,pD, nodat);
      write(f, 'weighted.index.e = ');
      escridatLIST(f, rx,preci, TRUE, FALSE);

      b:= PolaPondera(FALSE, Pa,pD, nodat);
      write(f, 'weighted.index.p = ');
      escridatLIST(f, b,preci, FALSE, TRUE);
      writeln(f, ');');
    end;

  DelVectoR(pD, nodat);
End;

{ ********************************************************************* }


END. 
