{ =====================================================
 Copyright (C) 2007 Francisco A. Ocana & Pablo Onate 

 This file is part of IndElec

 IndElec is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 IndElec is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You have received a copy of the GNU General Public License
 in a file called COPYING which you should have received with
 IndElec. If not, see <http://www.gnu.org/licenses/>.
 ===================================================== 
}
unit VOLFORMU;

interface

uses COMUN,  VOLDEF;

{
 This unit contains the formulae of the total volatility index (fTV),
 the bloc volatility index (fBV) and the within-bloc volatility index.
 Further, some corrections providing an approximation to the total volatility index
 is included here. 

 The notation here used comes from the article by Ocaa (2007)

Ocana FA (2007). "An Approximation Problem in Computing Electoral Volatility". 
 Applied Mathematics and Computation, Vol. 192, No. 2, pg. 299-310. 
 doi:10.1016/j.amc.2007.03.032.
}


{ Corrections to approximate the total volatility measure
}

procedure aprVolBM(var Vm, sumPer2, sumPer1: Tnumero);
procedure aprVolVA(var Vm, sumPer2, sumPer1: Tnumero);
procedure aprVolThet(var VLm, sumPer2, sumPer1: Tnumero; THETA: Tnumero0);


{ Total volatility index 
}
function fTV(Pvol: PVecVolati;  dimVol: TRgPartido;
             VOTOS: boolean): Tnumero;

{ Bloc volatility index 
}
function fBV(Pvol: PVecVolati;  dimVol: TRgPartido;
             var vSimbo: TSimbAfili;  NoBloqVol: byte;
             VOTOS: boolean): Tnumero;



implementation



procedure aprVolBM(var Vm, sumPer2, sumPer1: Tnumero);
{ 
This procedure computes the Bartolini and Mair correction given by

  V[m] + 0.5 |(100-sumPer2) - (100-sumPer1)| = V[m] + 0.5*|sumPer1 - sumPer2|
}
Begin
  Vm := Vm + 0.5*ABS(sumPer2 - sumPer1);
End;
{ *************************************************************** }


procedure aprVolVA(var Vm, sumPer2, sumPer1: Tnumero);
{ 
This procedure computes the Ocana (2007) correction which is given by

 V[m] + 0.5*max[ (100-sumPer2), (100-sumPer1)] =
 V[m] + 0.5*[100- min[sumPer2, sumPer1] ]
}
var rx: Tnumero;
Begin
  rx:= sumPer1;
  if (sumPer2 < sumPer1) then rx:= sumPer2;

  Vm:= Vm + 0.5*(100-rx);
End;

{ *************************************************************** }

procedure aprVolThet(var VLm, sumPer2, sumPer1: Tnumero; THETA: Tnumero0);
{ 
 This procedure computes a correction of the class of volatility approximations
 proposed by Ocana (2007), which are denoted by V_theta

 VL[m] + THETA*min[ (100-sumPer2), (100-sumPer1)] =
 VL[m] + THETA*[100 - max[sumPer2, sumPer1] ]

 where THETA belongs to the interval [0,1].

}

  var rx: Tnumero;

Begin
  rx:= sumPer2;
  if (sumPer2 < sumPer1) then rx:= sumPer1;

  VLm:= VLm + 0.5*ABS(sumPer2 - sumPer1) + THETA*(100-rx);
End;

{ *************************************************************** }

function fTV(Pvol: PVecVolati; dimVol: TRgPartido;
             VOTOS: boolean): Tnumero;

{
  The Total Volatility index is computed by considering its electoral version,
 if VOTOS = TRUE, or its parliamentary version, otherwise.
}

  var i: TRgPartido;
      ac: Tnumero;
Begin
  ac:= 0;

  if VOTOS
  then
   for i:= 0 to dimVol do ac:= ac + ABS(PVol^[i].Vp)
  else
   for i:= 0 to dimVol do ac:= ac + ABS(PVol^[i].Ep);

  fTV:= ac * 0.5;
End;
{ *************************************************************** }

function fBV(Pvol: PVecVolati; dimVol: TRgPartido;
             var vSimbo: TSimbAfili; NoBloqVol: byte;
             VOTOS: boolean): Tnumero;

{
  The Bloc Volatility index is computed by considering its electoral version,
 if VOTOS = TRUE, or its parliamentary version, otherwise.
 The number of blocs is given by NoBloqVol+1.
}

  var i: TRgPartido;
      j: byte;
      vac: array[0..MaxNoBloqsVol] of Tnumero;
      r: Tnumero;
Begin
  FillChar(vac, Succ(MaxNoBloqsVol) * sizeOf(Tnumero), 0);

  for i:= 0 to dimVol do
    begin
      if VOTOS then r:= PVol^[i].Vp
               else r:= PVol^[i].Ep;

      for j:= 0 to NoBloqVol do
        if PVol^[i].cbloque = vSimbo[j] then
          begin
            vac[j] := vac[j] + r;
            break;
          end;
    end;

  r:= 0;
  for j:= 0 to NoBloqVol do r:= r + ABS( vac[j] );

  fBV:= 0.5*r;
End;


End.
